#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>    
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>	
    [Description("Colors bars with a gradient based on volume over a user specified lookback period")]
    public class VolumeGradientBars : Indicator
    {
        #region Variables
        
		private int lookBack = 100;
		private Color highVolume = Color.Lime;
		private Color lowVolume = Color.Red;
		
		private SMA sma;
		private StdDev std;
		
        #endregion
		
        protected override void Initialize()
        {
            Overlay				= true;
        }
		
		protected override void OnStartUp()
		{
			sma = SMA(Volume,lookBack);
			std = StdDev(Volume,lookBack);
		}

        protected override void OnBarUpdate()
        {
			if (CurrentBar<lookBack)
				return;
						
			double devs = (Volume[0]-sma[0])/std[0];
			devs = Math.Min(3,Math.Max(-2,devs));
			devs += 2;
			
			double value = devs==0 ? 0 : devs/5;			
			BarColor = BlendColors(value);
        }
		
		public Color BlendColors(double amount)
		{
			byte r = (byte) ((highVolume.R * amount) + lowVolume.R * (1 - amount));
			byte g = (byte) ((highVolume.G * amount) + lowVolume.G * (1 - amount));
			byte b = (byte) ((highVolume.B * amount) + lowVolume.B * (1 - amount));
			return Color.FromArgb(r, g, b);
		}		
		
        #region Properties
        [Description("")]
		[GridCategory("Parameters")]		
		[XmlIgnore()]
        public int LookBack
        {
            get { return lookBack; }
            set { lookBack = Math.Max(1,value); }
        }
		
		[XmlIgnore()]
		[Description("")]
		[GridCategory("Parameters")]		
		public Color HighVolume
		{
			get { return highVolume; }
			set { highVolume = value; }
		}
		
		[Browsable(false)]
		public string HighVolumeS
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(highVolume); }
			set { highVolume = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
		[Description("")]
		[GridCategory("Parameters")]		
		public Color LowVolume
		{
			get { return lowVolume; }
			set { lowVolume = value; }
		}
		
		[Browsable(false)]
		public string LowVolumeS
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lowVolume); }
			set { lowVolume = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VolumeGradientBars[] cacheVolumeGradientBars = null;

        private static VolumeGradientBars checkVolumeGradientBars = new VolumeGradientBars();

        /// <summary>
        /// Colors bars with a gradient based on volume over a user specified lookback period
        /// </summary>
        /// <returns></returns>
        public VolumeGradientBars VolumeGradientBars(Color highVolume, int lookBack, Color lowVolume)
        {
            return VolumeGradientBars(Input, highVolume, lookBack, lowVolume);
        }

        /// <summary>
        /// Colors bars with a gradient based on volume over a user specified lookback period
        /// </summary>
        /// <returns></returns>
        public VolumeGradientBars VolumeGradientBars(Data.IDataSeries input, Color highVolume, int lookBack, Color lowVolume)
        {
            if (cacheVolumeGradientBars != null)
                for (int idx = 0; idx < cacheVolumeGradientBars.Length; idx++)
                    if (cacheVolumeGradientBars[idx].HighVolume == highVolume && cacheVolumeGradientBars[idx].LookBack == lookBack && cacheVolumeGradientBars[idx].LowVolume == lowVolume && cacheVolumeGradientBars[idx].EqualsInput(input))
                        return cacheVolumeGradientBars[idx];

            lock (checkVolumeGradientBars)
            {
                checkVolumeGradientBars.HighVolume = highVolume;
                highVolume = checkVolumeGradientBars.HighVolume;
                checkVolumeGradientBars.LookBack = lookBack;
                lookBack = checkVolumeGradientBars.LookBack;
                checkVolumeGradientBars.LowVolume = lowVolume;
                lowVolume = checkVolumeGradientBars.LowVolume;

                if (cacheVolumeGradientBars != null)
                    for (int idx = 0; idx < cacheVolumeGradientBars.Length; idx++)
                        if (cacheVolumeGradientBars[idx].HighVolume == highVolume && cacheVolumeGradientBars[idx].LookBack == lookBack && cacheVolumeGradientBars[idx].LowVolume == lowVolume && cacheVolumeGradientBars[idx].EqualsInput(input))
                            return cacheVolumeGradientBars[idx];

                VolumeGradientBars indicator = new VolumeGradientBars();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.HighVolume = highVolume;
                indicator.LookBack = lookBack;
                indicator.LowVolume = lowVolume;
                Indicators.Add(indicator);
                indicator.SetUp();

                VolumeGradientBars[] tmp = new VolumeGradientBars[cacheVolumeGradientBars == null ? 1 : cacheVolumeGradientBars.Length + 1];
                if (cacheVolumeGradientBars != null)
                    cacheVolumeGradientBars.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheVolumeGradientBars = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Colors bars with a gradient based on volume over a user specified lookback period
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumeGradientBars VolumeGradientBars(Color highVolume, int lookBack, Color lowVolume)
        {
            return _indicator.VolumeGradientBars(Input, highVolume, lookBack, lowVolume);
        }

        /// <summary>
        /// Colors bars with a gradient based on volume over a user specified lookback period
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumeGradientBars VolumeGradientBars(Data.IDataSeries input, Color highVolume, int lookBack, Color lowVolume)
        {
            return _indicator.VolumeGradientBars(input, highVolume, lookBack, lowVolume);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Colors bars with a gradient based on volume over a user specified lookback period
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumeGradientBars VolumeGradientBars(Color highVolume, int lookBack, Color lowVolume)
        {
            return _indicator.VolumeGradientBars(Input, highVolume, lookBack, lowVolume);
        }

        /// <summary>
        /// Colors bars with a gradient based on volume over a user specified lookback period
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumeGradientBars VolumeGradientBars(Data.IDataSeries input, Color highVolume, int lookBack, Color lowVolume)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VolumeGradientBars(input, highVolume, lookBack, lowVolume);
        }
    }
}
#endregion
